/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <QTemporaryFile>
#include <QDir>
#include <blkid/blkid.h>
#include <unistd.h>

struct DiskDevice {
    QString     Device;
    QString     UUID;
    QString     Label;
    QString     Type;
};

quint32     iKey;
QString     aKey;
QFile       inFile;
QFile       outFile;

int writeline(QFile *file, const char * data, int len) {
    if (file->write(data) != len)
        return -1;
    if (file->write("\n") != 1)
        return -1;
    return 0;
}

bool probeDevice(blkid_dev dev) {

    const char          *tag;
    const char          *value;
    QString             device;
    blkid_tag_iterate   iter;
    QString             uuid;
    QString             type;
    QString             label;
    QString             ws;
    QString             wso;

    device = blkid_dev_devname(dev);
    iter = blkid_tag_iterate_begin(dev);
    if (! iter)
        return false;
    while (blkid_tag_next(iter, &tag, &value) == 0) {
        ws = tag;
        if (ws == "UUID") {
            uuid = value;
            uuid = uuid.trimmed();
        }
        if (ws == "LABEL") {
            label = value;
            label = label.trimmed();
        }
        if (ws == "TYPE") {
            type = value;
            type = type.trimmed();
        }
    }
    wso = device + "\t" + type + "\t" + uuid + "\t" + label;
    if (writeline(&outFile, wso.toLocal8Bit(), wso.length()) < 0) {
        return false;
    }
    blkid_tag_iterate_end(iter);
    return true;
}

int scan() {

    blkid_cache         cache = NULL;
    blkid_dev_iterate   iter;
    blkid_dev           dev;

    if (blkid_get_cache(&cache, NULL) < 0)
        return -1;
    if (blkid_probe_all(cache) < 0)
        return -1;
    iter = blkid_dev_iterate_begin(cache);
    if (! iter)
        return -1;
    while (blkid_dev_next(iter, &dev) == 0) {
        dev = blkid_verify(cache, dev);
        if (! dev)
            continue;
        if (! probeDevice(dev)) {
            blkid_dev_iterate_end(iter);
            return -1;
        }
    }
    blkid_dev_iterate_end(iter);
    return 0;
}

int copy() {

    QString             inLine;
    QString             fileName;
    QFile::Permissions  perms = QFile::ReadUser | QFile::WriteUser |
                        QFile::ReadGroup | QFile::ReadOther;

#ifdef QT_DEBUG
    fileName = "/tmp/fstab";
    QTemporaryFile tempFile("/tmp/.fisysmgr-XXXXXX");
#else
    fileName = "/etc/fstab";
    QTemporaryFile tempFile("/etc/.fisysmgr-XXXXXX");
#endif
    if (! tempFile.open())
        return -1;
    if (! tempFile.setPermissions(perms)) {
        tempFile.remove();
        return -1;
    }
    inLine = inFile.readLine();
    if (inLine == "") {
        tempFile.remove();
        return -1;
    }
    inLine = inLine.trimmed();
    while (inLine != "EOF") {
        if (writeline(&tempFile, inLine.toLocal8Bit().data(), inLine.length()) < 0) {
            tempFile.remove();
            return - 1;
        }
        inLine = inFile.readLine();
        if (inLine == "") {
            tempFile.remove();
            return -1;
        }
        inLine = inLine.trimmed();
    }
    tempFile.close();
    if (rename(tempFile.fileName().toLocal8Bit().data(), fileName.toLocal8Bit().data()) != 0)
        return -1;
    return 0;
}

int main() {

    int         result;
    QString     ws;
    QStringList wsl;

    inFile.setFileName("/dev/urandom");
    if (! inFile.open(QIODevice::ReadOnly))
        return -1;
    result = inFile.read((char *)&iKey,4);
    if (result != 4) {
        inFile.close();
        return -1;
    }
    inFile.close();
    if (! outFile.open(stdout, QIODevice::WriteOnly))
        return -1;
    aKey = QString::number(iKey,16);
    if (writeline(&outFile, aKey.toLocal8Bit().data(), aKey.length()) < 0) {
        outFile.close();
        return -1;
    }
    outFile.flush();
    if (! inFile.open(stdin, QIODevice::ReadOnly))
        return -1;
    ws = inFile.readLine().trimmed();
    result = 0;
    while (ws != "close") {
        wsl = ws.split(":");
        if (wsl[0] == aKey) {
            if (wsl[1] == "copy") {
                result = copy();
                if (result != 0) {
                    if (writeline(&outFile, "failed", 6) < 0) {
                        result = -1;
                        break;
                    }
                } else {
                    if (writeline(&outFile, "success", 7) < 0) {
                        result = -1;
                        break;
                    }
                }
                outFile.flush();
            }
            if (wsl[1] == "scan") {
                result = scan();
                if (result !=0) {
                    if (writeline(&outFile, "failed", 6) < 0) {
                        result = -1;
                        break;
                    }
                } else {
                    if (writeline(&outFile, "EOF", 3) < 0) {
                        result = -1;
                        break;
                    }
                }
                outFile.flush();
            }
            ws = inFile.readLine().trimmed();
        } else {
           result = -1;
           break;
        }
    }
    inFile.close();
    outFile.close();
    return result;
}
