/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "browsedlg.h"
#include "ui_browsedlg.h"
#include <QMessageBox>

BrowseDlg::BrowseDlg(QWidget *parent) : QDialog(parent), ui(new Ui::BrowseDlg) {

    QHBoxLayout *FSLayout;
    QFont       font;

    ui->setupUi(this);
    loading = false;
    FSList = new FSTableView(this);
    FSList->setEditTriggers(QAbstractItemView::NoEditTriggers);
    FSList->setTabKeyNavigation(false);
    FSList->setDropIndicatorShown(false);
    FSList->setDragDropOverwriteMode(false);
    FSList->setSelectionMode(QAbstractItemView::SingleSelection);
    FSList->setSelectionBehavior(QAbstractItemView::SelectRows);
    FSList->setShowGrid(false);
    FSList->setWordWrap(false);
    FSList->setCornerButtonEnabled(false);
    FSList->verticalHeader()->setVisible(false);
#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
    FSList->horizontalHeader()->setResizeMode(QHeaderView::Interactive);
#else
    FSList->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
#endif
    font = FSList->font();
    font.setFamily("Monospace");
    FSList->setFont(font);
    FSLayout = new QHBoxLayout();
    FSLayout->setContentsMargins(0, 0, 0, 0);
    FSLayout->addWidget(FSList);
    QWidget::setTabOrder(ui->leStatus, FSList);
    ui->FSFrame->setLayout(FSLayout);
    fslModel = new TableModel(this, 4);
    sortProxy.setSourceModel(fslModel);
    FSList->setModel(&sortProxy);
    ui->rbLocal = ui->rbLocal;
    ui->rbSamba = ui->rbSamba;
    ui->rbNFS = ui->rbNFS;
    ui->rbUUID = ui->rbUUID;
    ui->rbLabel = ui->rbLabel;
    ui->rbDevice = ui->rbDevice;
    ui->tbRefresh->setIcon(QIcon::fromTheme("view-refresh", QIcon(":/icons/view-refresh.png")));
    ui->pbApply->setIcon(QIcon::fromTheme("dialog-ok-apply", QIcon(":/icons/dialog-ok-apply.png")));
    ui->pbCancel->setIcon(QIcon::fromTheme("dialog-cancel", QIcon(":/icons/dialog-ok.png")));
    on_rbLocal_clicked();
    connect(FSList, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(fslist_dblclicked()));
    connect(FSList, SIGNAL(currentItemChanged(const QModelIndex &, const QModelIndex &)),
            this, SLOT(currentItemChanged(const QModelIndex &, const QModelIndex &)));
}

BrowseDlg::~BrowseDlg() {
    delete ui;
}

void BrowseDlg::value(QString *device, QString *type) const {

    int         curRow;
    QModelIndex CurItem;

    curRow = FSList->currentIndex().row();
    if (ui->rbLocal->isChecked()) {
        CurItem = sortProxy.index(curRow, 1);
        *type = sortProxy.data(CurItem).toString();
        if (ui->rbUUID->isChecked()) {
            CurItem = sortProxy.index(curRow, 2);
            *device = "UUID=" + sortProxy.data(CurItem).toString();
        }
        if (ui->rbLabel->isChecked()) {
            CurItem = sortProxy.index(curRow, 3);
            *device = "LABEL=" + sortProxy.data(CurItem).toString();
        }
        if (ui->rbDevice->isChecked()) {
            CurItem = sortProxy.index(curRow, 0);
            *device = sortProxy.data(CurItem).toString();
        }
    }
    if (ui->rbSamba->isChecked()) {
        *type = "cifs";
        CurItem = sortProxy.index(curRow, 1);
        *device = "//" + sortProxy.data(CurItem).toString();
        CurItem = sortProxy.index(curRow, 2);
        *device += "/" + sortProxy.data(CurItem).toString();
    }
    if (ui->rbNFS->isChecked()) {
        *type = "nfs";
        CurItem = sortProxy.index(curRow, 0);
        *device = sortProxy.data(CurItem).toString();
    }
}

void BrowseDlg::reloadFS() {

    int         winWidth;
    int         winHeight;
    int         result;
    CredsDialog credsDialog;

    loading = true;
    setCursor(QCursor(Qt::WaitCursor));
    if (ui->rbLocal->isChecked()) {
        ui->pbApply->setEnabled(false);
        loadDisk();
    }
    if (ui->rbSamba->isChecked()) {
        winWidth = settings->value("CredsDialog/Width", 300).toInt();
        winHeight = settings->value("CredsDialog/Height", 200).toInt();
        credsDialog.resize(winWidth, winHeight);
        credsDialog.setValue(curCreds);
        result = credsDialog.exec();
        if ((winWidth != credsDialog.width()) || (winHeight != credsDialog.height())) {
            winWidth = credsDialog.width();
            winHeight = credsDialog.height();
            if (winWidth < 300)
                winWidth = 300;
            if (winHeight < 210)
                winHeight = 210;
            settings->setValue("CredsDialog/Width", winWidth);
            settings->setValue("CredsDialog/Height", winHeight);
        }
        if (result) {
            if (curCreds != credsDialog.value()) {
                curCreds = credsDialog.value();
                settings->setValue("CIFS/Credentials", curCreds);
            }
            workgroup = credsDialog.workgroup();
            username = credsDialog.username();
            password = credsDialog.password();
            ui->pbApply->setEnabled(false);
            loadSamba();
        }
    }
    if (ui->rbNFS->isChecked()) {
        ui->pbApply->setEnabled(false);
        loadNFS();
    }
    setCursor(QCursor(Qt::ArrowCursor));
    loading = false;
}

void BrowseDlg::loadDisk() {
    if (! diskInfo.refresh()) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to load local disk information"));
        return;
    };
    showDisk();
}

void BrowseDlg::showDisk() {

    QModelIndex itemIdx;
    int         rowcount;

    rowcount = fslModel->rowCount();
    fslModel->removeRows(0, rowcount);
    if (diskInfo.devCount() > 0) {
        for (int idx = 0; idx < diskInfo.devCount(); idx++) {
            fslModel->insertRow(idx);
            diskDev = diskInfo[idx];
            itemIdx = fslModel->index(idx, 0);
            fslModel->setData(itemIdx, diskDev.Device);
            itemIdx = fslModel->index(idx, 1);
            fslModel->setData(itemIdx, diskDev.Type);
            itemIdx = fslModel->index(idx, 2);
            fslModel->setData(itemIdx, diskDev.UUID);
            itemIdx = fslModel->index(idx, 3);
            fslModel->setData(itemIdx, diskDev.Label);
        }
    }
    if (fslModel->rowCount() > 0) {
        sortProxy.invalidate();
        sortProxy.setSortMode(local);
        sortProxy.sort(0);
        ui->pbApply->setEnabled(true);
    }
}

void BrowseDlg::loadSamba() {
    switch(cifsInfo.scanMode()) {
        case 0 :
            wrkGroup = "";
            server = "";
            break;
        case SMBC_WORKGROUP :
            server = "";
            break;
    }
    cifsInfo.refresh(wrkGroup,server);
    if (cifsInfo.numNewItems() == 0) {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("No Items found"));
    }
    ui->leStatus->setText("");
    showSamba();
}

void BrowseDlg::showSamba() {

    QModelIndex itemIdx;
    int         rowcount;

    rowcount = fslModel->rowCount();
    fslModel->removeRows(0, rowcount);
    if (cifsInfo.shareCount() > 0) {
        for (int idx = 0; idx < cifsInfo.shareCount(); idx++) {
            fslModel->insertRow(idx);
            cShare = (cifsInfo)[idx];
            itemIdx = fslModel->index(idx, 0);
            fslModel->setData(itemIdx, cShare.workgroup);
            itemIdx = fslModel->index(idx, 1);
            fslModel->setData(itemIdx, cShare.server);
            itemIdx = fslModel->index(idx, 2);
            fslModel->setData(itemIdx, cShare.share);
            itemIdx = fslModel->index(idx, 3);
            fslModel->setData(itemIdx, cShare.comment);
        }
    }
    if (fslModel->rowCount() > 0) {
        sortProxy.invalidate();
        sortProxy.setSortMode(samba);
        sortProxy.sort(0);
    }
}

void BrowseDlg::loadNFS() {
    showNFS();
}

void BrowseDlg::showNFS() {

    int rowcount;

    rowcount = fslModel->rowCount();
    fslModel->removeRows(0, rowcount);
}

void BrowseDlg::currentItemChanged(const QModelIndex & current, const QModelIndex &) {

    int         curRow;
    QModelIndex curItem;
    QString     wg;
    QString     fs;
    QString     sh;

    if (loading)
        return;
    curRow = current.row();
    if (ui->rbSamba->isChecked()) {
        curItem = sortProxy.index(curRow, 0);
        wg = sortProxy.data(curItem).toString();
        curItem = sortProxy.index(curRow, 1);
        fs = sortProxy.data(curItem).toString();
        curItem = sortProxy.index(curRow, 2);
        sh = sortProxy.data(curItem).toString();
        if ((wg != "") && (fs == "")) {
            wrkGroup = wg;
            server = "";
            cifsInfo.setScanMode(SMBC_WORKGROUP);
            ui->tbScan->setEnabled(true);
            ui->leStatus->setText(wg);
            ui->pbApply->setEnabled(false);
        }
        if ((wg != "") && (fs != "")) {
            wrkGroup = wg;
            server = fs;
            cifsInfo.setScanMode(SMBC_SERVER);
            ui->tbScan->setEnabled(true);
            ui->leStatus->setText(fs);
            ui->pbApply->setEnabled(false);
        }
        if (sh != "") {
            cifsInfo.setScanMode(SMBC_FILE_SHARE);
            ui->tbScan->setEnabled(false);
            ui->leStatus->setText("");
            ui->pbApply->setEnabled(true);
        }
    }
    if (ui->rbLocal->isChecked()) {
        if (ui->rbLocal->isChecked()) {
            curItem = sortProxy.index(curRow, 2);
            if (sortProxy.data(curItem).toString() != "") {
                ui->rbUUID->setEnabled(true);
            } else {
                if (ui->rbUUID->isChecked()) {
                    if (ui->rbLabel->isEnabled()) {
                        ui->rbLabel->setChecked(true);
                    } else {
                        ui->rbDevice->setChecked(true);
                    }
                }
                ui->rbUUID->setEnabled(false);
            }
            curItem = sortProxy.index(curRow, 3);
            if (sortProxy.data(curItem).toString() != "") {
                ui->rbLabel->setEnabled(true);
            } else {
                if (ui->rbLabel->isChecked()) {
                    if (ui->rbUUID->isEnabled()) {
                        ui->rbUUID->setChecked(true);
                    } else {
                        ui->rbDevice->setChecked(true);
                    }
                }
                ui->rbLabel->setEnabled(false);
            }
        }
    }
}

void BrowseDlg::on_pbApply_clicked() {
    accept();
}

void BrowseDlg::on_pbCancel_clicked() {
    reject();
}

void BrowseDlg::on_tbRefresh_clicked() {
    cifsInfo.setScanMode(0);
    reloadFS();
    FSList->resizeRowsToContents();
    FSList->resizeColumnsToContents();
    if (fslModel->rowCount() > 0)
        FSList->selectRow(0);
    FSList->setFocus();
}

void BrowseDlg::on_tbScan_clicked() {

    int curRow;

    curRow = FSList->currentIndex().row();
    reloadFS();
    FSList->resizeRowsToContents();
    FSList->resizeColumnsToContents();
    if (curRow > 0) {
        FSList->selectRow(curRow);
    } else {
        if (fslModel->rowCount() > 0)
            FSList->selectRow(0);
    }
    FSList->setFocus();
}

void BrowseDlg::fslist_dblclicked() {
    if (cifsInfo.scanMode() == SMBC_FILE_SHARE)
        return;
    on_tbScan_clicked();
}

void BrowseDlg::on_rbLocal_clicked() {
    cifsInfo.setScanMode(0);
    ui->pbApply->setEnabled(false);
    ui->gbFSID->setEnabled(true);
    ui->tbScan->setEnabled(false);
    loading = true;
    fslModel->setHeaderData(0, Qt::Horizontal, tr("Device "));
    fslModel->setHeaderData(1, Qt::Horizontal, tr("FS Type "));
    fslModel->setHeaderData(2, Qt::Horizontal, tr("UUID "));
    fslModel->setHeaderData(3, Qt::Horizontal, tr("Label "));
    showDisk();
    loading = false;
    ui->leStatus->setText("");
    if (fslModel->rowCount() > 0)
        FSList->selectRow(0);
    FSList->resizeRowsToContents();
    FSList->resizeColumnsToContents();
    FSList->setFocus();
}

void BrowseDlg::on_rbSamba_clicked() {
    ui->pbApply->setEnabled(false);
    ui->gbFSID->setEnabled(false);
    ui->tbScan->setEnabled(true);
    loading =true;
    fslModel->setHeaderData(0, Qt::Horizontal, tr("Workgroup "));
    fslModel->setHeaderData(1, Qt::Horizontal, tr("Server "));
    fslModel->setHeaderData(2, Qt::Horizontal, tr("Share "));
    fslModel->setHeaderData(3, Qt::Horizontal, tr("Comment "));
    showSamba();
    loading = false;
    ui->leStatus->setText("NETWORK");
    if (fslModel->rowCount() > 0)
        FSList->selectRow(0);
    FSList->resizeRowsToContents();
    FSList->resizeColumnsToContents();
    FSList->setFocus();
}

void BrowseDlg::on_rbNFS_clicked(){
    cifsInfo.setScanMode(0);
    ui->tbScan->setEnabled(false);
    loading =true;
    showNFS();
    loading = false;
    ui->leStatus->setText("");
}
