/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "cifsinfo.h"

int Cifsinfo::scanModeVal;

Cifsinfo::Cifsinfo(QObject *parent) : QObject(parent) {
    context = 0;
}

Cifsinfo::~Cifsinfo() {
}

bool Cifsinfo::refresh(const QString wg, const QString server) {

    bool                status;
    int                 rowCount;
    CifsShare           csw;

    if (scanModeVal == 0) {
        while (! cifsShares.isEmpty())
            cifsShares.removeFirst();
    }
    context = smbc_new_context();
    if (!context)
        return false;
    smbc_setDebug(context, 0);
    smbc_setFunctionAuthData(context, credentials);
    smbc_setOptionUseKerberos(context, 1);
    smbc_setOptionFallbackAfterKerberos(context, 1);
    if (!smbc_init_context(context)) {
        smbc_free_context(context, 0);
        return false;
    }
    smbc_set_context(context);
    status = scanDir(wg,server);
    if (nNewItems != 0) {
        switch (scanModeVal) {
            case SMBC_WORKGROUP :
                rowCount = cifsShares.count() - 1;
                for (int idx = rowCount; idx >= 0; idx--) {
                    csw = cifsShares[idx];
                    if ((csw.workgroup == wg) && (csw.server == "")) {
                        cifsShares.removeAt(idx);
                    }
                }
                break;
            case SMBC_SERVER :
                rowCount = cifsShares.count() - 1;
                for (int idx = rowCount; idx >= 0; idx--) {
                    csw = cifsShares[idx];
                    if ((csw.workgroup == wg) && (csw.server == server) &&
                            (csw.share == "")) {
                        cifsShares.removeAt(idx);
                    }
                }
                break;
        }
    }
    smbc_free_context(context, 0);
    if (! status)
        return false;
    return true;
}

int Cifsinfo::shareCount() const {
    return cifsShares.size();
}

int Cifsinfo::numNewItems() const {
    return nNewItems;
}

int Cifsinfo::scanMode() const {
    return scanModeVal;
}

void Cifsinfo::setScanMode(int mode) {
    scanModeVal = mode;
}

const CifsShare & Cifsinfo::operator[] (int idx) const {
    return cifsShares[idx];
}

void Cifsinfo::credentials(const char *, const char *,
    char *domain, int wgmaxlen, char *user, int unmaxlen,
    char *pass, int pwmaxlen) {
    strncpy(user, username.toLocal8Bit().data(), unmaxlen - 1);
    strncpy(pass, password.toLocal8Bit().data(), pwmaxlen - 1);
    strncpy(domain, workgroup.toLocal8Bit().data(), wgmaxlen - 1);
}

bool Cifsinfo::scanDir(const QString wg, const QString server) {

    int             dirHandle;
    smbc_dirent     *dirEntry;
    QString         directory;
    CifsShare       csw;
    int             entryType;

    if (scanModeVal == 0) {
        directory = "smb://";
    } else {
        if (scanModeVal == SMBC_WORKGROUP) {
            directory = "smb://" + wg;
        } else {
            if (scanModeVal == SMBC_SERVER) {
                directory = "smb://" + server;
            }
        }
    }
    nNewItems = 0;
    dirHandle = smbc_opendir(directory.toLocal8Bit().data());
    if (dirHandle < 0) {
        return false;
    }
    while ((dirEntry = smbc_readdir(dirHandle)) != NULL) {
        entryType = dirEntry->smbc_type;
        if (entryType == SMBC_WORKGROUP) {
            csw.workgroup = dirEntry->name;
            csw.server = "";
            csw.share = "";
            csw.comment = "";
            cifsShares.append(csw);
            nNewItems++;
        }
        if (entryType == SMBC_SERVER) {
            csw.workgroup = wg;
            csw.server = dirEntry->name;
            csw.share = "";
            csw.comment = "";
            cifsShares.append(csw);
            nNewItems++;
        }
        if (entryType == SMBC_FILE_SHARE) {
            if (strstr(dirEntry->name, "$") == NULL) {
                csw.workgroup = wg;
                csw.server = server;
                csw.share = dirEntry->name;
                csw.comment = dirEntry->comment;
                cifsShares.append(csw);
                nNewItems++;
            }
        }
    }
    smbc_closedir(dirHandle);
    return true;
}
