/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "credsdialog.h"
#include "ui_credsdialog.h"

CredsDialog::CredsDialog(int mode, QWidget *parent) : QDialog(parent), ui(new Ui::CredsDialog) {

    QDir        credsDir;
    QStringList fileList;
    QModelIndex index;
    int         count;

    loading = true;
    ui->setupUi(this);
    if (mode == 0) {
        ui->pbClose->hide();
    } else {
        ui->pbOK->hide();
        ui->pbDlgCancel->hide();
    }
    ui->pbOK->setIcon(QIcon::fromTheme("dialog-ok-apply", QIcon(":/icons/dialog-ok-apply.png")));
    ui->pbDlgCancel->setIcon(QIcon::fromTheme("dialog-cancel", QIcon(":/icons/dialog-ok.png")));
    ui->pbClose->setIcon(QIcon::fromTheme("application-exit", QIcon(":/icons/application-exit.png")));
    fnRegExp.setPattern("[^/]*");
    fnValidator.setRegExp(fnRegExp);
    sortProxy = new QSortFilterProxyModel(ui->cbCredentials);
    sortProxy->setSortCaseSensitivity(Qt::CaseInsensitive);
    sortProxy->setSourceModel(ui->cbCredentials->model());
    ui->cbCredentials->model()->setParent(sortProxy);
    ui->cbCredentials->setModel(sortProxy);
    dirName = QDir::homePath() + "/.fisysmgr/";
    credsDir.setPath(dirName);
    if (! credsDir.exists())
        return;
    credsDir.setFilter(QDir::Files);
    fileList = credsDir.entryList();
    count = fileList.count();
    if (count == 0)
        return;
    ui->pbOK->setEnabled(true);
    ui->pbEdit->setEnabled(true);
    ui->pbDelete->setEnabled(true);
    ui->leFilename->setValidator(&fnValidator);
    for (int idx = 0; idx < count; idx ++) {
        ui->cbCredentials->model()->insertRow(0);
        index = ui->cbCredentials->model()->index(0,0);
        ui->cbCredentials->model()->setData(index, fileList[idx]);
    }
    if (count > 0) {
        sortProxy->invalidate();
        ui->cbCredentials->model()->sort(0, Qt::AscendingOrder);
    }
    loading = false;
    editMode = false;
    QTimer::singleShot(0,this, SLOT(loadCreds()));
}

CredsDialog::~CredsDialog() {
    delete ui;
}

QString CredsDialog::value() {

    int current;

    current = ui->cbCredentials->currentIndex();
    return dirName + ui->cbCredentials->itemText(current);
}

QString CredsDialog::workgroup() {
    return ui->leDomain->text();
}

QString CredsDialog::username() {
    return ui->leUserName->text();
}

QString CredsDialog::password() {
    return ui->lePassword->text();
}

void CredsDialog::setValue(QString val) {

    int idx;

    idx = ui->cbCredentials->findText(val.section("/",-1,-1));
    if (idx >= 0) {
        ui->cbCredentials->setCurrentIndex(idx);
        loadCreds();
    }
}

void CredsDialog::loadCreds() {

    int         current;
    QString     filename;
    QFile       credFile;
    QString     inLine;
    QStringList fields;

    if(loading)
        return;
    current = ui->cbCredentials->currentIndex();
    filename = dirName + ui->cbCredentials->itemText(current);
    credFile.setFileName(filename);
    if (! credFile.open(QIODevice::ReadOnly)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Unable to open credentials file ") + "'" + filename + "'.");
        close();
        return;
    }
    while (! credFile.atEnd()) {
        inLine = credFile.readLine().trimmed();
        fields = inLine.split('=');
        if (fields[0] == "domain")
            ui->leDomain->setText(fields[1]);
        if (fields[0] == "username")
            ui->leUserName->setText(fields[1]);
        if (fields[0] == "password")
            ui->lePassword->setText(fields[1]);
    }
    credFile.close();
}

void CredsDialog::on_pbOK_clicked() {
    accept();
}

void CredsDialog::on_pbDlgCancel_clicked() {
    reject();
}

void CredsDialog::on_pbEdit_clicked() {
    ui->pbOK->setEnabled(false);
    ui->pbEdit->setEnabled(false);
    ui->pbSave->setEnabled(true);
    ui->pbNewFile->setEnabled(false);
    ui->pbDelete->setEnabled(false);
    ui->pbCancel->setEnabled(true);
    ui->cbCredentials->setEnabled(false);
    ui->leDomain->setEnabled(true);
    ui->lePassword->setEnabled(true);
    ui->leUserName->setEnabled(true);
    ui->leDomain->setFocus();
    editMode = true;
}

void CredsDialog::on_pbSave_clicked() {

    QTemporaryFile      tempFile;
    QFile::Permissions  perms = QFile::ReadUser | QFile::WriteUser;
    QString             domain;
    QString             username;
    QString             password;
    QString             filename;
    int                 current;
    QString             path;
    QDir                directory;
    QModelIndex         index;

    domain = ui->leDomain->text().trimmed();
    username = ui->leUserName->text().trimmed();
    password = ui->lePassword->text().trimmed();
    if (username == "") {
        QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Username is required!"));
        ui->leUserName->setFocus();
        return;
    }
    if (password == "") {
        QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Password is required!"));
        ui->leUserName->setFocus();
        return;
    }
    if (! editMode) {
        filename = ui->leFilename->text().trimmed();
        if (filename == "") {
            QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Credential Name is required!"));
            ui->leUserName->setFocus();
            return;
        }
    }
    path = QDir::homePath() + "/.fisysmgr";
    if (! directory.mkpath(path)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to create crendentials directory."));
        tempFile.remove();
        return;
    }
    tempFile.setFileTemplate(path + "/Creds-XXXXXX");
    if (! tempFile.open()) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to create temporary file."));
        return;
    }
    if (! tempFile.setPermissions(perms)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to set permissions on temporary file."));
        tempFile.remove();
        return;
    }
    if (domain != "") {
        if (! writeline(&tempFile, "domain=" + domain)) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to write to temporary file."));
            tempFile.remove();
            return;
        }
    }
    if (! writeline(&tempFile, "username=" + username)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to write to temporary file."));
        tempFile.remove();
        return;
    }
    if (! write(&tempFile, "password=" + password)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to write to temporary file."));
        tempFile.remove();
        return;
    }
    tempFile.close();
    if (editMode) {
        current = ui->cbCredentials->currentIndex();
        filename = ui->cbCredentials->itemText(current);
    }
    filename = path + "/" + filename;
    if (rename(tempFile.fileName().toLocal8Bit().data(), filename.toLocal8Bit().data()) != 0) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to save crendentials file."));
        tempFile.remove();
        return;
    }
    if (! editMode) {
        loading = true;
        ui->cbCredentials->model()->insertRow(0);
        index = ui->cbCredentials->model()->index(0,0);
        ui->cbCredentials->model()->setData(index, ui->leFilename->text());
        sortProxy->invalidate();
        ui->cbCredentials->model()->sort(0, Qt::AscendingOrder);
        loading = false;
        current = ui->cbCredentials->findText(ui->leFilename->text());
        if (current >= 0)
            ui->cbCredentials->setCurrentIndex(current);
        ui->leFilename->setText("");
    }
    ui->pbSave->setEnabled(false);
    ui->pbNewFile->setEnabled(true);
    ui->pbDelete->setEnabled(true);
    ui->pbCancel->setEnabled(false);
    ui->cbCredentials->setEnabled(true);
    ui->leDomain->setEnabled(false);
    ui->lePassword->setEnabled(false);
    ui->leUserName->setEnabled(false);
    ui->pbEdit->setEnabled(true);
    ui->pbEdit->setFocus();
    ui->pbOK->setEnabled(true);
}

bool CredsDialog::write(QFile *file, QString data) {
    if (file->write(data.toLocal8Bit().data()) != data.length())
        return false;
    return true;
}

bool CredsDialog::writeline(QFile *file, QString data) {
    if (file->write(data.toLocal8Bit().data()) != data.length())
        return false;
    if (file->write("\n") != 1)
        return false;
    return true;
}

void CredsDialog::on_pbCancel_clicked() {
    ui->pbSave->setEnabled(false);
    ui->pbNewFile->setEnabled(true);
    ui->pbDelete->setEnabled(true);
    ui->pbCancel->setEnabled(false);
    ui->cbCredentials->setEnabled(true);
    ui->leDomain->setEnabled(false);
    ui->lePassword->setEnabled(false);
    ui->leUserName->setEnabled(false);
    ui->leFilename->setEnabled(false);
    ui->leFilename->setText("");
    loadCreds();
    ui->pbEdit->setEnabled(true);
    ui->pbEdit->setFocus();
    ui->pbOK->setEnabled(true);
}

void CredsDialog::on_pbNewFile_clicked() {
    ui->pbOK->setEnabled(false);
    ui->pbEdit->setEnabled(false);
    ui->pbSave->setEnabled(true);
    ui->pbNewFile->setEnabled(false);
    ui->pbDelete->setEnabled(false);
    ui->pbCancel->setEnabled(true);
    ui->cbCredentials->setEnabled(false);
    ui->leDomain->setText("");
    ui->leUserName->setText("");
    ui->lePassword->setText("");
    ui->leFilename->setText("");
    ui->leDomain->setEnabled(true);
    ui->lePassword->setEnabled(true);
    ui->leUserName->setEnabled(true);
    ui->leFilename->setEnabled(true);
    ui->leDomain->setFocus();
    editMode = false;
}

void CredsDialog::on_pbDelete_clicked() {

    int     result;
    QString msg;
    QString filename;
    QString name;
    int     current;
    QMessageBox::StandardButtons buttons;

    buttons = QMessageBox::Yes | QMessageBox::No;
    current = ui->cbCredentials->currentIndex();
    name = ui->cbCredentials->itemText(current);
    msg = tr("Remove Credentials for '") + name + "' ?";
    result = QMessageBox::warning(this, tr("FISysMgr - Confirm"), msg, buttons);
    if (result == QMessageBox::No)
        return;
    filename = dirName + name;
    if (! QFile::remove(filename)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to remove crendentials file '") + name + "'");
    }
    ui->cbCredentials->model()->removeRow(current);
    if (ui->cbCredentials->count() == 0) {
        ui->pbDelete->setEnabled(false);
        ui->pbOK->setEnabled(false);
        return;
    }
}

void CredsDialog::on_leDomain_editingFinished() {
    ui->leDomain->setText(ui->leDomain->text().trimmed());
    ui->leDomain->setText(ui->leDomain->text().toUpper());
}

void CredsDialog::on_leUserName_editingFinished() {
    ui->leUserName->setText(ui->leUserName->text().trimmed());
}

void CredsDialog::on_cbCredentials_currentIndexChanged(int) {
    if (ui->cbCredentials->count()) {
        loadCreds();
    } else {
        ui->leDomain->setText("");
        ui->leFilename->setText("");
        ui->lePassword->setText("");
        ui->leUserName->setText("");
    }
}

void CredsDialog::on_pbClose_clicked() {
    close();
}
