/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "editdialog.h"
#include "ui_editdialog.h"
#include "QFileDialog"
#include "browsedlg.h"
#include "optionsdlg.h"

EditDialog::EditDialog(QWidget *parent) : QDialog(parent), ui(new Ui::EditDialog) {
    ui->setupUi(this);
    ui->pbApply->setIcon(QIcon::fromTheme("dialog-ok-apply", QIcon(":/icons/dialog-ok-apply.png")));
    ui->pbCancel->setIcon(QIcon::fromTheme("dialog-cancel", QIcon(":/icons/dialog-ok.png")));
}

EditDialog::~EditDialog() {
    delete ui;
}

QVariantList EditDialog::value() const {

    QVariantList wd;

    wd.append(ui->leLocation->text());
    wd.append(ui->leMntPoint->text());
    wd.append(ui->leType->text());
    wd.append(ui->leOptions->text());
    wd.append(ui->spDump->value());
    wd.append(ui->spCheck->value());
    return wd;
}

void EditDialog::setValue(const QVariantList* data) {
    ui->leLocation->setText(data->at(0).toString());
    ui->leMntPoint->setText(data->at(1).toString());
    ui->leType->setText(data->at(2).toString());
    ui->leOptions->setText(data->at(3).toString());
    ui->spDump->setValue(data->at(4).toInt());
    ui->spCheck->setValue(data->at(5).toInt());
}

void EditDialog::on_tbFSLookup_clicked() {

    int         winWidth;
    int         winHeight;
    int         result;
    QString     device;
    QString     type;
    BrowseDlg   browseDlg(this);

    winWidth = settings->value("BrowseDialog/Width", 400).toInt();
    winHeight = settings->value("BrowseDialog/Height", 400).toInt();
    browseDlg.resize(winWidth, winHeight);
    result = browseDlg.exec();
    if ((winWidth != browseDlg.width()) || (winHeight != browseDlg.height())) {
        settings->setValue("BrowseDialog/Width", browseDlg.width());
        settings->setValue("BrowseDialog/Height", browseDlg.height());
    }
    if (result) {
        browseDlg.value(&device, &type);
        ui->leLocation->setText(device);
        if (ui->leType->text() != type)
            ui->leOptions->setText("");
        ui->leType->setText(type);
    }
}

void EditDialog::on_tbOptions_clicked() {

    int         winWidth;
    int         winHeight;
    int         result;
    OptionsDlg  *optsDlg;

    optsDlg = new OptionsDlg(this, ui->leType->text());
    winWidth = settings->value("OptionsDialog/Width", 420).toInt();
    winHeight = settings->value("OptionsDialog/Height", 320).toInt();
    optsDlg->resize(winWidth, winHeight);
    optsDlg->setValue(ui->leOptions->text());
    result = optsDlg->exec();
    if ((winWidth != optsDlg->width()) || (winHeight != optsDlg->height())) {
        settings->setValue("OptionsDialog/Width", optsDlg->width());
        settings->setValue("OptionsDialog/Height", optsDlg->height());
    }
    if (result)
        ui->leOptions->setText(optsDlg->value());
    delete optsDlg;
}

void EditDialog::on_tbMntPoint_clicked() {

    int         result;
    int         winWidth;
    int         winHeight;
    QFileDialog dirDialog(this);
    QString     dirName;

    winWidth = settings->value("MountDialog/Width", 640).toInt();
    winHeight = settings->value("MountDialog/Height", 480).toInt();
    dirDialog.resize(winWidth, winHeight);
    dirDialog.setOption(QFileDialog::DontUseNativeDialog);
    dirDialog.setFileMode(QFileDialog::Directory);
    dirDialog.setDirectory(ui->leMntPoint->text());
    result = dirDialog.exec();
    if ((winWidth != dirDialog.width()) || (winHeight != dirDialog.height())) {
        winWidth = dirDialog.width();
        winHeight = dirDialog.height();
        if (winWidth < 640)
            winWidth = 640;
        if (winHeight < 480)
            winHeight =480;
        settings->setValue("MountDialog/Width", winWidth);
        settings->setValue("MountDialog/Height", winHeight);
    }
    if (result) {
        dirName = dirDialog.selectedFiles()[0];
        if (dirName.length() > 1) {
            if (dirName[dirName.length() - 1] == '/') {
                dirName = dirName.left(dirName.length() - 1);
            }
        }
        ui->leMntPoint->setText(dirName);
    }
}

void EditDialog::on_pbApply_clicked() {
    if (ui->leLocation->text() == "") {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("File System Location is required."));
        ui->tbFSLookup->setFocus();
        return;
    }
    if (ui->leType->text() == "") {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("File System Type is required."));
        ui->leType->setFocus();
        return;
    }
    if (ui->leMntPoint->text() == "") {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("Mount Point is required."));
        ui->tbMntPoint->setFocus();
        return;
    }
    if (ui->leOptions->text() == "") {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("At least one option is required."));
        ui->tbOptions->setFocus();
        return;
    }
    accept();
}

void EditDialog::on_pbCancel_clicked() {
    reject();
}
