﻿/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "fsmmain.h"
#include "ui_fsmmain.h"
#include "editdialog.h"
#include <QTimer>
#include <unistd.h>
#include <QFileDialog>

QString FsmMain::fstabHash = "5fb9a04bf237c44085070a924d8cf4a4d5816db7";

FsmMain::FsmMain(QWidget *parent) : QMainWindow(parent), ui(new Ui::FsmMain) {

    QString     ws;

    startupOK = true;
    settings = new QSettings(this);
    sPlayer = new SoundPlayer(this);
    helperProc = new QProcess(this);
    ws = "fisysmgr-c0dd6785-2bda-44d8-b979-5bc66c9aca2d";
    localSocket.connectToServer(ws);
    if (localSocket.waitForConnected(500)) {
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("An instance of FiSysMgr is already started."));
        startupOK = false;
        QTimer::singleShot(0, this, SLOT(startup()));
        return;
    }
    localServer = new QLocalServer(this);
    connect(localServer, SIGNAL(newConnection()), this, SLOT(newLocalConnection()));
    if (! localServer->listen(ws)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to create local server"));
        startupOK = false;
        QTimer::singleShot(0, this, SLOT(startup()));
        return;
    }
    hProcEnv = QProcessEnvironment::systemEnvironment();
#ifdef QT_DEBUG
    hProcEnv.insert("PATH", hProcEnv.value("PATH") +
               ":/home/barry/Projects/QtProjects/fisysmgr/build-fisysmgrprj-Qt_4_8_6_System_2-Debug/helper/");
#endif
    helperProc->setProcessEnvironment(hProcEnv);
    helperProc->setReadChannel(QProcess::StandardOutput);
    helperProc->start("pkexec fisysmgr-helper");
    if (! helperProc->waitForStarted()) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"), tr("Unable to start helper process"));
        startupOK = false;
        QTimer::singleShot(0, this, SLOT(startup()));
        return;
    }
    accessKey = "";
    while ((! helperProc->canReadLine()) && (helperProc->state() == QProcess::Running)) {
        helperProc->waitForReadyRead();
    }
    if (helperProc->canReadLine())
        accessKey = helperProc->readLine().trimmed();
    if (accessKey == "") {
        startupOK = false;
        QTimer::singleShot(0, this, SLOT(startup()));
        return;
    }
    ui->setupUi(this);
    ui->tbUp->setIcon(QIcon::fromTheme("go-up", QIcon(":/icons/go-up.png")));
    ui->tbDown->setIcon(QIcon::fromTheme("go-down", QIcon(":/icons/go-down.png")));
    ui->tbEdit->setIcon(QIcon::fromTheme("document-edit", QIcon(":/icons/document-edit.png")));
    ui->tbNew->setIcon(QIcon::fromTheme("document-new", QIcon(":/icons/document-new.png")));
    ui->tbDelete->setIcon(QIcon::fromTheme("edit-delete", QIcon(":/icons/edit-delete.png")));
    ui->pbSave->setIcon(QIcon::fromTheme("document-save", QIcon(":/icons/document-save.png")));
    ui->pbClose->setIcon(QIcon::fromTheme("application-exit", QIcon(":/icons/application-exit.png")));
    fstabView = ui->fstabView;
    fstabView->setFocus();
    fstabModel = new TableModel(this, 6);
    filterProxy.setFilterRegExp("^\\s*[\\x21-\\x22\\x24-\\x7E].*$");
    filterProxy.setFilterKeyColumn(0);
    filterProxy.setFilterCaseSensitivity(Qt::CaseInsensitive);
    filterProxy.setSourceModel(fstabModel);
    fstabView->setModel(&filterProxy);
    fstabModel->setHeaderData(0, Qt::Horizontal, tr("File System "));
    fstabModel->setHeaderData(1, Qt::Horizontal, tr("Mount Point "));
    fstabModel->setHeaderData(2, Qt::Horizontal, tr("Type "));
    fstabModel->setHeaderData(3, Qt::Horizontal, tr("Options "));
    fstabModel->setHeaderData(4, Qt::Horizontal, tr("Dump "));
    fstabModel->setHeaderData(5, Qt::Horizontal, tr("fsck "));
    username = qgetenv("USER");
    winWidth = settings->value("MainWindow/Width", 640).toInt();
    winHeight = settings->value("MainWindow/Height", 480).toInt();
    curCreds = settings->value("CIFS/Credentials", "").toString();
    resize(winWidth, winHeight);
#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
    fstabView->horizontalHeader()->setResizeMode(QHeaderView::Interactive);
#else
    fstabView->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
#endif
    fstabView->resizeColumnsToContents();
    QTimer::singleShot(0, this, SLOT(startup()));
}

FsmMain::~FsmMain() {
    if (startupOK) {
        winWidth = settings->value("MainWindow/Width", 640).toInt();
        winHeight = settings->value("MainWindow/Height", 480).toInt();
        if ((winWidth != width()) || (winHeight != height())) {
            settings->setValue("MainWindow/Width", width());
            settings->setValue("MainWindow/Height", height());
        }
    }
    if (helperProc->state() == QProcess::Running) {
        if (helperProc->write("close\n") != 6) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::warning(this,tr("FiSysMg - Message"),
                                 tr("Unable to write 'close' to helper process"));
        }
        if (! helperProc->waitForFinished()) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Unable to close helper process"));
        }
    }
    delete ui;
}

void FsmMain::startup() {
    if (! startupOK) {
        close();
        return;
    }
    diskInfo.setAccessKey(accessKey);
    if (! loadfstab()) {
            close();
            return;
    }
    if (settings->value("Messages/BackupDlg", true).toBool()) {
        backupDlg = new BackupDlg(this);
        backupDlg->exec();
        if (backupDlg->disableChecked() > 0) {
            settings->setValue("Messages/BackupDlg", false);
        }
        delete backupDlg;
    }
}

bool FsmMain::loadfstab() {

    int         numColumns;
    int         idy;
    int         entryCnt;
    QString     ws;
    QString     line;
    QModelIndex itemIndex;
    QRegExp     comment("^\\s*#.*$|^\\s*$", Qt::CaseInsensitive);
    QRegExp     whiteSpace("\\s+", Qt::CaseInsensitive);
    QStringList fields;

#ifdef QT_DEBUG
    ws = "/tmp/fstab";
#else
    ws = "/etc/fstab";
#endif
    fsFile.setFileName(ws);
    if (! fsFile.open(QIODevice::ReadOnly)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Unable to open file ") + "'" + ws + "'" );
        return false;
    }
    fsStream = new QTextStream(&fsFile);
    idy = 0;
    entryCnt = 0;
    while (! fsStream->atEnd()) {
        line = fsStream->readLine();
        if (line.contains(comment)) {
            fstabModel->insertRow(idy);
            itemIndex = fstabModel->index(idy, 0);
            fstabModel->setData(itemIndex, line);
        } else {
            entryCnt += 1;
            fields = line.split(whiteSpace);
            for (int idx = 0; idx < 3; idx++) {
                fields.replace(idx, fields.value(idx).replace("\\040"," "));
                fields.replace(idx, fields.value(idx).replace("\\040"," "));
            }
            fstabModel->insertRow(idy);
            numColumns = fields.count();
            if (numColumns > 6)
                numColumns = 6;
            for (int idx = 0; idx < numColumns; idx++) {
                itemIndex = fstabModel->index(idy, idx);
                fstabModel->setData(itemIndex, fields[idx]);
            }
        }
        idy++;
    }
    fsFile.close();
    delete fsStream;
    if (entryCnt > 0) {
        ui->tbEdit->setEnabled(true);
        ui->pbSave->setEnabled(true);
        filterProxy.invalidate();
        fstabView->selectRow(0);
    }
    fstabView->resizeRowsToContents();
    fstabView->resizeColumnsToContents();
    return true;
}

void FsmMain::newLocalConnection() {
    raise();
}

void FsmMain::on_action_About_triggered() {

    AboutBox about(this);

    about.exec();
}

void FsmMain::on_actionBackup_FileSystem_Config_triggered() {

    int         result;
    int         winWidth;
    int         winHeight;
    QFileDialog fileDialog(this);
    QString     fileName;
    QString     inLine;

    winWidth = settings->value("BackupDialog/Width", 640).toInt();
    winHeight = settings->value("BackupDialog/Height", 480).toInt();
    fileDialog.resize(winWidth, winHeight);
    fileDialog.setOption(QFileDialog::DontUseNativeDialog);
    fileDialog.setAcceptMode(QFileDialog::AcceptSave);
    fileDialog.setDirectory("/home/barry");
    fileDialog.setFileMode(QFileDialog::AnyFile);
    fileDialog.selectFile("fstab-" + QDate::currentDate().toString("yyMMdd"));
    result = fileDialog.exec();
    if ((winWidth != fileDialog.width()) || (winHeight != fileDialog.height())) {
        winWidth = fileDialog.width();
        winHeight = fileDialog.height();
        if (winWidth < 640)
            winWidth = 640;
        if (winHeight < 480)
            winHeight =480;
        settings->setValue("BackupDialog/Width", winWidth);
        settings->setValue("BackupDialog/Height", winHeight);
    }
    if (! result)
        return;
    fileName = fileDialog.selectedFiles()[0];
#ifdef QT_DEBUG
    fstabFile.setFileName("/tmp/fstab");
#else
    fstabFile.setFileName("/etc/fstab");
#endif
    saveFile.setFileName(fileName);
    if (! fstabFile.open(QIODevice::ReadOnly|QIODevice::Text)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to open fstab file!"));
        return;
    }
    if (! saveFile.open(QIODevice::WriteOnly|QIODevice::Truncate|QIODevice::Text)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to create backup file!"));
        fstabFile.close();
        return;
    }
    if (saveFile.write(fstabHash.toLocal8Bit().data()) != fstabHash.length()) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write to backup file!"));
        saveFile.close();
        fstabFile.close();
        return;
    }
    if (saveFile.write("\n") != 1) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write to backup file!"));
        saveFile.close();
        fstabFile.close();
        return;
    }
    while (! fstabFile.atEnd()) {
        inLine =  fstabFile.readLine();
        if (inLine == "") {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::critical(this,tr("FiSysMg - Message"),
                                  tr("Unable to read fstab file!"));
            saveFile.close();
            fstabFile.close();
            return;
        }
        if (saveFile.write(inLine.toLocal8Bit().data()) != inLine.length()) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::critical(this,tr("FiSysMg - Message"),
                                  tr("Unable to write to backup file!"));
            saveFile.close();
            fstabFile.close();
            return;
        }
    }
    QMessageBox::information(this,tr("FiSysMg - Message"),
                          tr("fstab backed up successfully."));
    fstabFile.close();
    saveFile.close();
}

void FsmMain::on_actionR_estore_Filesystem_Config_triggered() {

    int         result;
    int         winWidth;
    int         winHeight;
    QFileDialog fileDialog(this);
    QString     fileName;
    QString     ws;
    QStringList fileList;
    int         lineCount;
    int         timeout;

    winWidth = settings->value("RestoreDialog/Width", 640).toInt();
    winHeight = settings->value("RestoreDialog/Height", 480).toInt();
    fileDialog.resize(winWidth, winHeight);
    fileDialog.setOption(QFileDialog::DontUseNativeDialog);
    fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
    fileDialog.setDirectory("/home/barry");
    fileDialog.setFileMode(QFileDialog::ExistingFile);
    result = fileDialog.exec();
    if ((winWidth != fileDialog.width()) || (winHeight != fileDialog.height())) {
        winWidth = fileDialog.width();
        winHeight = fileDialog.height();
        if (winWidth < 640)
            winWidth = 640;
        if (winHeight < 480)
            winHeight =480;
        settings->setValue("RestoreDialog/Width", winWidth);
        settings->setValue("RestoreDialog/Height", winHeight);
    }
    if (! result)
        return;
    fileName = fileDialog.selectedFiles()[0];
    saveFile.setFileName(fileName);
    if (! saveFile.open(QIODevice::ReadOnly|QIODevice::Text)) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to open backup file!"));
        return;
    }
    ws = saveFile.readLine();
    if (ws == "") {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to read backup file!"));
        saveFile.close();
        return;
    }
    ws = ws.trimmed();
    if (ws != fstabHash) {
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              fileName + tr(" Does not seem to be a vaild backup file!"));
        saveFile.close();
        return;
    }
    while (! saveFile.atEnd()) {
        ws = saveFile.readLine();
        if (ws == "") {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::critical(this,tr("FiSysMg - Message"),
                                  tr("Unable to read backup file!"));
            saveFile.close();
            return;
        }
        fileList.append(ws);
    }
    saveFile.close();
    if (helperProc->write(accessKey.toLocal8Bit().data()) != accessKey.length()) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write to helper process!"));
        return;
    }
    if (helperProc->write(":copy\n") != 6) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write 'copy' to helper process!"));
        return;
    }
    lineCount = fileList.count();
    for (int idy = 0; idy < lineCount; idy++) {
        if (helperProc->write(fileList[idy].toLocal8Bit().data()) != fileList[idy].length()) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::critical(this,tr("FiSysMg - Message"),
                                  tr("Unable to write file to helper process!"));
            return;
        }
    }
    if (helperProc->write("EOF\n") != 4) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write 'EOF' to helper process!"));
        return;
    }
    timeout = 0;
    ws = "";
    do {
        if ((helperProc->state() != QProcess::Running) || (timeout > 300)) {
            ws = "failed";
            break;
        } else {
            if (helperProc->canReadLine()) {
                ws = helperProc->readLine().trimmed();
            } else {
                helperProc->waitForReadyRead(10);
                usleep(100000);
            }
        }
        timeout++;
    } while (ws == "");
    if (ws == "success") {
        fstabModel->removeRows(0,fstabModel->rowCount());
        if (! loadfstab()) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::warning(this,tr("FiSysMg - Message"), tr("fstab restored but unable to reload it."));
           close();
           return;
        }
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("fstab restored successfully."));
    } else {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Unable to restore fstab."));
    }
}

void FsmMain::on_actionE_xIT_triggered() {
    close();
}

void FsmMain::on_fstabView_doubleClicked() {
    on_tbEdit_clicked();
}

void FsmMain::on_tbUp_clicked() {

    int         rowcnt;
    int         currow;
    QModelIndex ItemIdx;

    rowcnt = filterProxy.rowCount();
    if (rowcnt > 1) {
        currow = fstabView->currentIndex().row();
        if (currow > 0) {
            for (int column = 5; column >= 0; column--)
                swapfields(currow, currow - 1, column);
            ItemIdx = filterProxy.index(currow - 1, 0);
            fstabView->setCurrentIndex(ItemIdx);
        }
    }
    fstabView->setFocus();
}

void FsmMain::on_tbDown_clicked() {

    int         rowcnt;
    int         currow;
    QModelIndex ItemIdx;

    rowcnt = filterProxy.rowCount();
    if (rowcnt > 1) {
        currow = fstabView->currentIndex().row();
        if (currow < (rowcnt - 1)) {
            for (int column = 5; column >= 0; column--)
                swapfields(currow, currow + 1, column);
            ItemIdx = filterProxy.index(currow + 1, 0);
            fstabView->setCurrentIndex(ItemIdx);
        }
    }
    fstabView->setFocus();
}

void FsmMain::on_tbEdit_clicked() {

    int             result;
    int             currow;
    QModelIndex     CurItemIdx;
    EditDialog      editDlg(this);
    QVariantList    data;

    currow = fstabView->currentIndex().row();
    for (int idx = 0; idx < 6; idx++) {
        CurItemIdx = filterProxy.index(currow, idx);
        data.append(filterProxy.data(CurItemIdx));
    }
    editDlg.setValue(&data);
    winWidth = settings->value("EditDialog/Width", 400).toInt();
    winHeight = settings->value("EditDialog/Height", 300).toInt();
    editDlg.resize(winWidth, winHeight);
    result = editDlg.exec();
    if ((winWidth != editDlg.width()) || (winHeight != editDlg.height())) {
        settings->setValue("EditDialog/Width", editDlg.width());
        settings->setValue("EditDialog/Height", editDlg.height());
    }
    if (result) {
        data = editDlg.value();
        for (int idx = 0; idx < 6; idx++) {
            CurItemIdx = filterProxy.index(currow, idx);
            filterProxy.setData(CurItemIdx, data[idx]);
        }
        fstabView->resizeRowsToContents();
        fstabView->resizeColumnsToContents();
    }
    fstabView->setFocus();
}

void FsmMain::on_tbNew_clicked() {

    int             result;
    int             currow;
    int             newrow;
    QModelIndex     CurItemIdx;
    EditDialog      editDlg(this);
    QVariantList    data;

    currow = fstabView->currentIndex().row();
    winWidth = settings->value("EditDialog/Width", 400).toInt();
    winHeight = settings->value("EditDialog/Height", 300).toInt();
    editDlg.resize(winWidth, winHeight);
    result = editDlg.exec();
    if ((winWidth != editDlg.width()) || (winHeight != editDlg.height())) {
        settings->setValue("EditDialog/Width", editDlg.width());
        settings->setValue("EditDialog/Height", editDlg.height());
    }
    if (result) {
        CurItemIdx = fstabView->currentIndex();
        newrow = filterProxy.mapToSource(CurItemIdx).row();
        newrow++;
        fstabModel->insertRow(newrow);
        data = editDlg.value();
        for (int idx = 0; idx < 6; idx++) {
            CurItemIdx = fstabModel->index(newrow, idx);
            fstabModel->setData(CurItemIdx, data[idx]);
        }
        filterProxy.invalidate();
        currow++;
        fstabView->selectRow(currow);
        fstabView->resizeRowsToContents();
        fstabView->resizeColumnsToContents();
        if (! ui->tbEdit->isEnabled()) {
            ui->tbEdit->setEnabled(true);
            ui->pbSave->setEnabled(true);
        }

    }
    fstabView->setFocus();
}

void FsmMain::on_tbDelete_clicked() {

    int         rowcnt;
    int         selrow;
    QModelIndex currow;
    QMessageBox::StandardButtons buttons;
    int         result;
    QString     msg;
    QString     fileSystem;
    QString     mountPoint;

    selrow = fstabView->currentIndex().row();
    currow = filterProxy.index(selrow, 0);
    fileSystem = filterProxy.data(currow).toString();
    currow = filterProxy.index(selrow, 1);
    mountPoint = filterProxy.data(currow).toString();
    buttons = QMessageBox::Yes | QMessageBox::No;
    msg = tr("Remove entry for filesystem:\n'") + fileSystem + "'\nMountpoint: '" + mountPoint + "' ?";
    result = QMessageBox::warning(this, tr("FISysMgr - Confirm"), msg, buttons);
    if (result == QMessageBox::No) {
        return;
    }
    rowcnt = filterProxy.rowCount();
    if (rowcnt > 0) {
        currow = fstabView->currentIndex();
        selrow = currow.row();
        currow = filterProxy.mapToSource(currow);
        fstabModel->removeRow(currow.row());
        rowcnt = filterProxy.rowCount();
        if (rowcnt > 0) {
            if (selrow >= rowcnt)
                selrow = rowcnt - 1;
            currow = fstabView->model()->index(selrow,0);
            fstabView->setCurrentIndex(currow);
        } else {
            ui->tbEdit->setEnabled(false);
            ui->pbSave->setEnabled(false);
        }
    }
    fstabView->setFocus();
}

void FsmMain::on_pbSave_clicked() {

    int                 idx;
    int                 idy;
    int                 entryCnt;
    int                 timeout;
    QModelIndex         itemIndex;
    QRegExp             comment("^\\s*#.*$|^\\s*$", Qt::CaseInsensitive);
    QString             ws;
    QString             outLine;

    if (helperProc->write(accessKey.toLocal8Bit().data()) != accessKey.length()) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write to helper process!"));
        return;
    }
    if (helperProc->write(":copy\n") != 6) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write 'copy' to helper process!"));
        return;
    }
    entryCnt = fstabModel->rowCount();
    for (idy = 0; idy < entryCnt; idy++) {
        outLine = "";
        for (idx = 0; idx < 6; idx++) {
            itemIndex = fstabModel->index(idy, idx);
            ws = fstabModel->data(itemIndex).toString();
            if (idx == 0) {
                if (ws.contains(comment)) {
                    outLine += ws;
                    break;
                } else {
                    ws.replace(" ", "\\040");
                    outLine += ws;
                }
            } else {
                if (idx < 4)
                    ws.replace(" ", "\\040");
                outLine += "    " + ws;
            }
        }
        outLine += "\n";
        if (helperProc->write(outLine.toLocal8Bit().data()) != outLine.length()) {
            sPlayer->play("qrc:///sounds/Warning.ogg");
            QMessageBox::critical(this,tr("FiSysMg - Message"),
                                  tr("Unable to write to helper process!"));
            return;
        }
    }
    if( helperProc->write("EOF\n") != 4) {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::critical(this,tr("FiSysMg - Message"),
                              tr("Unable to write 'EOF' to helper process!"));
        return;
    }
    timeout = 0;
    ws = "";
    do {
        if ((helperProc->state() != QProcess::Running) || (timeout > 300)) {
            ws = "failed";
            break;
        } else {
            if (helperProc->canReadLine()) {
                ws = helperProc->readLine().trimmed();
            } else {
                helperProc->waitForReadyRead(10);
                usleep(100000);
            }
        }
        timeout++;
    } while (ws == "");
    if (ws == "success") {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("Changes saved successfully."));
    } else {
        sPlayer->play("qrc:///sounds/Warning.ogg");
        QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Unable to save changes."));
    }
}

void FsmMain::on_pbClose_clicked() {
    close();
}

void FsmMain::swapfields(int row, int row1, int column) {

    QModelIndex CurItemIdx;
    QModelIndex NextItemIdx;
    QVariant    CurRow;
    QVariant    NextRow;

    CurItemIdx = filterProxy.index(row, column);
    NextItemIdx = filterProxy.index(row1, column);
    CurRow = filterProxy.data(CurItemIdx);
    NextRow = filterProxy.data(NextItemIdx);
    filterProxy.setData(CurItemIdx, NextRow);
    filterProxy.setData(NextItemIdx, CurRow);
}

void FsmMain::on_action_Manage_Credentials_triggered() {

    CredsDialog credsDialog(1, this);

    winWidth = settings->value("CredsDialog/Width", 300).toInt();
    winHeight = settings->value("CredsDialog/Height", 200).toInt();
    credsDialog.resize(winWidth, winHeight);
    credsDialog.setValue(curCreds);
    credsDialog.exec();
    if ((winWidth != credsDialog.width()) || (winHeight != credsDialog.height())) {
        winWidth = credsDialog.width();
        winHeight = credsDialog.height();
        if (winWidth < 300)
            winWidth = 300;
        if (winHeight < 210)
            winHeight = 210;
        settings->setValue("CredsDialog/Width", winWidth);
        settings->setValue("CredsDialog/Height", winHeight);
    }
}
