/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "optionsdlg.h"
#include "ui_optionsdlg.h"
#include "paramdlg.h"

OptionsDlg::OptionsDlg(QWidget *parent, QString type) : QDialog(parent), ui(new Ui::OptionsDlg) {

    int         numcbs;
    OptChkBox   *chkBox;

    ui->setupUi(this);
    fstype = type;
    layout = new QGridLayout();
    buildInterface();
    ui->frOptions->setLayout(layout);
    numcbs = ui->frOptions->children().count();
    for (int idy = 0; idy < numcbs; idy++) {
        chkBox = qobject_cast<OptChkBox*>((ui->frOptions->children())[idy]);
        if (chkBox) {
            connect(chkBox, SIGNAL(toggled(bool)), this, SLOT(chkBoxToggled(bool)));
        }
    }
    ui->pbApply->setIcon(QIcon::fromTheme("dialog-ok-apply", QIcon(":/icons/dialog-ok-apply.png")));
    ui->pbCancel->setIcon(QIcon::fromTheme("dialog-cancel", QIcon(":/icons/dialog-ok.png")));
    updating = false;
    loading = false;
    editing = false;
}

OptionsDlg::~OptionsDlg() {
    delete ui;
}

void OptionsDlg::buildInterface() {

    bool    UseDefaults;
    QFrame  *line;

    cboxRow = 0;
    cboxCol = 0;
    UseDefaults = ((fstype != "swap") && (fstype != "cifs"));
    if (UseDefaults) {
        resource.setFileName(":/options/allfilesys");
        resource.open(QIODevice::ReadOnly);
        loadChkBoxes();
        resource.close();
        ui->cbDefaults->setEnabled(true);
        cboxRow++;
        cboxCol = 0;
        line = new QFrame(this);
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);
        layout->addWidget(line, cboxRow, cboxCol, 1, 4);
        cboxRow++;
    }
    resource.setFileName(":/options/" + fstype);
    resource.open(QIODevice::ReadOnly);
    loadChkBoxes();
    resource.close();
}

void OptionsDlg::loadChkBoxes() {

    QString     ws;
    QString     tltip;
    OptChkBox   *chkBox;

    chkBox = 0;
    while (! resource.atEnd()) {
        ws = resource.readLine().trimmed();
        if (ws != "") {
            if (ws[0] == '|') {
                ws = ws.right(ws.length() - 1);
                chkBox = new OptChkBox(this);
                chkBox->setObjectName("cb" + ws);
                chkBox->setText(ws);
                if (ws.contains("=")) {
                    chkBox->setToolTip(tr("If checked, right click to change value.\n\n"));
                }
                connect(chkBox, SIGNAL(rightClicked(OptChkBox*)), this, SLOT(rightClicked(OptChkBox*)));
                layout->addWidget(chkBox, cboxRow, cboxCol);
                tltip = "";
            }
            if (ws[0] == '<') {
                ws = ws.right(ws.length() - 1);
                chkBox->setMinValue((ws.toInt()));
            }
            if (ws[0] == '>') {
                ws = ws.right(ws.length() - 1);
                chkBox->setMaxValue((ws.toInt()));
            }
            if (ws[0] == '[') {
                ws = ws.right(ws.length() - 1);
                chkBox->addValue(ws);
            }
            if (ws[0] == '{') {
                ws = ws.right(ws.length() - 1);
                if (tltip == "") {
                    tltip = ws;
                } else {
                    tltip += "\n" + ws;
                }
            }
            if (ws[0] == '}') {
                chkBox->setToolTip(chkBox->toolTip() + tltip);
                cboxCol++;
                if (cboxCol > 3) {
                    cboxCol = 0;
                    cboxRow++;
                }
            }
            if (ws[0] == '=') {
                if (ws[1] == 'i')
                    chkBox->setParamType(pint);
                if (ws[1] == 's')
                    chkBox->setParamType(pstr);
                if (ws[1] == 't')
                    chkBox->setParamType(pitm);
                if (ws[1] == 'o')
                    chkBox->setParamType(poct);
            }
        }
    }
}

void OptionsDlg::setValue(QString val) {

    int         numval;
    int         numcbs;
    int         index;
    bool        found;
    OptChkBox   *chkBox;
    QString     ws;
    QStringList wsl;
    QString     wsp;

    options = val.split(",",QString::SkipEmptyParts);
    options.removeDuplicates();
    numval = options.count();
    found = false;
    for (index = 0; index < numval; index++) {
        if (options[index] == "defaults") {
            found = true;
            break;
        }
    }
    if (found) {
        loading = true;
        ui->cbDefaults->setChecked(true);
        loading = false;
    }
    numval = options.count();
    ws = "";
    for (int idy = 0; idy < numval; idy++) {
        if (ws == "") {
            ws += options[idy];
        } else {
            ws += "," + options[idy];
        }
    }
    ui->leOptions->setText(ws);
    numcbs = ui->frOptions->children().count();
    for (int idx = 0; idx < numval; idx++) {
        updating = true;
        for (int idy = 0; idy < numcbs; idy++) {
            chkBox = qobject_cast<OptChkBox*>((ui->frOptions->children())[idy]);
            if (chkBox) {
                if (chkBox->paramType() == pnone) {
                    ws = options[idx];
                    wsp = "";
                } else {
                    wsl = options[idx].split("=");
                    if (wsl.count() > 1) {
                        ws = wsl[0] + "=";
                        wsp = wsl[1];
                    } else {
                        ws = wsl[0];
                        wsp = "";
                    }
                }
                if (ws == chkBox->text()) {
                    chkBox->setChecked(true);
                    chkBox->setParameter(wsp);
                }
            }
        }
        updating = false;
    }
}

QString OptionsDlg::value() {
    return ui->leOptions->text();
}

void OptionsDlg::setDefaults() {
    enableCbox("cbrw");
    enableCbox("cbsuid");
    enableCbox("cbdev");
    enableCbox("cbexec");
    enableCbox("cbauto");
    enableCbox("cbnouser");
    enableCbox("cbasync");
    if (! loading)
        options.insert(0, "defaults");
}

void OptionsDlg::unsetDefaults() {

    int     numvals;
    int     index;
    bool    found;

    disableCbox("cbrw");
    disableCbox("cbsuid");
    disableCbox("cbdev");
    disableCbox("cbexec");
    disableCbox("cbauto");
    disableCbox("cbnouser");
    disableCbox("cbasync");
    numvals = options.count();
    found = false;
    for (index = 0; index < numvals; index++) {
        if (options[index] == "defaults") {
            found = true;
            break;
        }
    }
    if (found)
        options.removeAt(index);
}

void OptionsDlg::enableCbox(QString cb) {

    int         idx;
    OptChkBox   *chkBox;

    chkBox = ui->frOptions->findChild<OptChkBox *>(cb);
    if (chkBox) {
        chkBox->setChecked(true);
        chkBox->setEnabled(false);
        idx = options.indexOf(chkBox->text());
        if (idx >= 0)
            options.removeAt(idx);
    }
}

void OptionsDlg::disableCbox(QString cb) {

    OptChkBox   *chkBox;

    chkBox = ui->frOptions->findChild<OptChkBox *>(cb);
    if (chkBox) {
        chkBox->setChecked(false);
        chkBox->setEnabled(true);
    }
}

void OptionsDlg::on_cbDefaults_toggled(bool checked) {

    int     idx;
    QString ws;

    updating = true;
    if (checked) {
        setDefaults();
    } else {
        unsetDefaults();
    }
    if (! loading) {
        ws = "";
        idx = options.count();
        for (int idy = 0; idy < idx; idy++) {
            if (ws == "") {
                ws += options[idy];
            } else {
                ws += "," + options[idy];
            }
        }
        ui->leOptions->setText(ws);
    }
    updating = false;
}

void OptionsDlg::rightClicked(OptChkBox *chkBox) {

    QString curValue;
    int     idx;
    bool    result;
    QString ws;

    if (! chkBox->isChecked())
        return;
    if (chkBox->paramType() != pnone) {
        editing = true;
        curValue = chkBox->text() + chkBox->parameter();
        if (chkBox->text() == "credentials=") {
            result = credentials(chkBox);
        } else {
            result = editParam(chkBox);
        }
        if (result) {
            idx = options.indexOf(curValue);
            if (idx >= 0)
                options[idx] = chkBox->text() + chkBox->parameter();
        } else {
            return;
        }
        ws = "";
        idx = options.count();
        for (int idy = 0; idy < idx; idy++) {
            if (ws == "") {
                ws += options[idy];
            } else {
                ws += "," + options[idy];
            }
        }
        ui->leOptions->setText(ws);
    }
}

void OptionsDlg::chkBoxToggled(bool checked) {

    int         idx;
    bool        result;
    OptChkBox   *chkBox;
    QString     ws;

    if (updating)
        return;
    editing = false;
    chkBox = (OptChkBox*)sender();
    if (checked) {
        if (chkBox->paramType() != pnone) {
            if (chkBox->text() == "credentials=") {
                result = credentials(chkBox);
            } else {
                result = editParam(chkBox);
            }
            if (result)
                options.append(chkBox->text() + chkBox->parameter());
        } else {
            options.append(chkBox->text());
        }
    } else {
        if (chkBox->paramType() != pnone) {
            idx = options.indexOf(chkBox->text() + chkBox->parameter());
        } else {
            idx = options.indexOf(chkBox->text());
        }
        if (idx >= 0) {
            options.removeAt(idx);
        }
    }
    ws = "";
    idx = options.count();
    for (int idy = 0; idy < idx; idy++) {
        if (ws == "") {
            ws += options[idy];
        } else {
            ws += "," + options[idy];
        }
    }
    ui->leOptions->setText(ws);
}

bool OptionsDlg::credentials(OptChkBox *chkBox) {

    bool        result;
    int         winWidth;
    int         winHeight;
    CredsDialog credsDialog(0, this);

    credsDialog.setValue(chkBox->parameter());
    winWidth = settings->value("CredsDialog/Width", 300).toInt();
    winHeight = settings->value("CredsDialog/Height", 200).toInt();
    credsDialog.resize(winWidth, winHeight);
    if (chkBox->parameter() != "") {
        credsDialog.setValue(chkBox->parameter());
        result = credsDialog.exec();
    } else {
        credsDialog.setValue(curCreds);
        result = credsDialog.exec();
    }
    if ((winWidth != credsDialog.width()) || (winHeight != credsDialog.height())) {
        winWidth = credsDialog.width();
        winHeight = credsDialog.height();
        if (winWidth < 300)
            winWidth = 300;
        if (winHeight < 210)
            winHeight = 210;
        settings->setValue("CredsDialog/Width", winWidth);
        settings->setValue("CredsDialog/Height", winHeight);
    }
    if (result) {
        chkBox->setParameter(credsDialog.value());
    } else {
        if (! editing)
            chkBox->setChecked(false);
    }
    return result;
}

bool OptionsDlg::editParam(OptChkBox *chkBox) {

    bool                result;
    int                 winWidth;
    int                 winHeight;
    ParamDlg            *paramDlg;
    QIntValidator       intValidator;
    QRegExpValidator    regexValidator;
    QRegExp             regex;

    paramDlg = 0;
    if (chkBox->paramType() == pint) {
        paramDlg = new ParamDlg(this, false);
        paramDlg->setLabel(chkBox->text());
        if ((chkBox->minValue() != 0) || (chkBox->maxValue() != 0)) {
            intValidator.setRange(chkBox->minValue(), chkBox->maxValue());
        } else {
            intValidator.setRange(0, INT_MAX);
        }
        paramDlg->setValidator(&intValidator);
    }
    if (chkBox->paramType() == pitm) {
        paramDlg = new ParamDlg(this, true);
        paramDlg->setcbValues(chkBox->cbValues());
        paramDlg->setLabel(chkBox->text());
    }
    if (chkBox->paramType() == poct) {
        paramDlg = new ParamDlg(this, false);
        regex.setPattern("[0-7]{3}");
        regexValidator.setRegExp(regex);
        paramDlg->setValidator(&regexValidator);
    }
    if (chkBox->paramType() == pstr) {
        paramDlg = new ParamDlg(this, false);
        paramDlg->setLabel(chkBox->text());
    }
    paramDlg->setValue(chkBox->parameter());
    paramDlg->setToolTip(chkBox->toolTip());
    winWidth = settings->value("ParamDialog/Width", 220).toInt();
    winHeight = settings->value("ParamDialog/Height", 100).toInt();
    paramDlg->resize(winWidth, winHeight);
    result = paramDlg->exec();
    if ((winWidth != paramDlg->width()) || (winHeight != paramDlg->height())) {
        settings->setValue("ParamDialog/Width", paramDlg->width());
        settings->setValue("ParamDialog/Height", paramDlg->height());
    }
    if (result) {
        switch(chkBox->paramType()) {
            case pint :
                chkBox->setParameter(QString::number(paramDlg->value().toInt()));
                break;
            default   :
                chkBox->setParameter(paramDlg->value());
        }
    } else {
        if (! editing)
            chkBox->setChecked(false);
    }
    delete paramDlg;
    return result;
}

void OptionsDlg::on_pbApply_clicked() {
    if (ui->leOptions->text() == "") {
        QMessageBox::information(this,tr("FiSysMg - Message"), tr("At least one option is required."));
        ui->leOptions->setFocus();
        return;
    }
    accept();
}

void OptionsDlg::on_pbCancel_clicked() {
    reject();
}

void OptionsDlg::on_leOptions_editingFinished() {

    int         numcbs;
    OptChkBox   *chkBox;
    QString     ws;

    ws = ui->leOptions->text().replace(" ", "");
    ui->cbDefaults->setChecked(false);
    numcbs = ui->frOptions->children().count();
    for (int idy = 0; idy < numcbs; idy++) {
        chkBox = qobject_cast<OptChkBox*>((ui->frOptions->children())[idy]);
        if (chkBox)
            chkBox->setChecked(false);
    }
    setValue(ws);
}
