/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "paramdlg.h"
#include "ui_paramdlg.h"

ParamDlg::ParamDlg(QWidget *parent, bool combobox) : QDialog(parent), ui(new Ui::ParamDlg) {
    ui->setupUi(this);
    ui->pbOk->setIcon(QIcon::fromTheme("dialog-ok", QIcon(":/icons/dialog-ok.png")));
    ui->pbCancel->setIcon(QIcon::fromTheme("dialog-cancel", QIcon(":/icons/dialog-ok.png")));
    cbox = combobox;
    if (cbox) {
        ui->leParam->hide();
    } else {
        ui->cbbParam->hide();
    }
}

ParamDlg::~ParamDlg(){
    delete ui;
}

QString ParamDlg::value() {
    QString ws;

    if (cbox) {
        ws = ui->cbbParam->itemText(ui->cbbParam->currentIndex());
        return ws;
    } else {
        return ui->leParam->text();
    }
}

void ParamDlg::setValue(QString val) {

    int idx;

    if (cbox) {
        idx = ui->cbbParam->findText(val);
        if (idx >= 0)
            ui->cbbParam->setCurrentIndex(idx);
    } else {
        ui->leParam->setText(val);
    }
}

void ParamDlg::setLabel(QString val) {
    ui->lParam->setText(val);
}

void ParamDlg::on_pbOk_clicked() {

    int     wi;
    QString ws;

    if (! cbox) {
        if (ui->leParam->validator() != 0) {
            ws = ui->leParam->text();
            if (ui->leParam->validator()->validate(ws, wi) != QValidator::Acceptable)
                return;
        }
        if (ui->leParam->text() == "") {
            QMessageBox::warning(this,tr("FiSysMg - Message"), tr("Option value is required."));
            ui->leParam->setFocus();
            return;
        }
    }
    accept();
}

void ParamDlg::on_pbCancel_clicked() {
    reject();
}

void ParamDlg::setValidator(QValidator *vld) {
    ui->leParam->setValidator(vld);
}

void ParamDlg::setcbValues(QStringList val) {

    int cnt;

    cnt = val.count();
    for (int idx = 0;idx < cnt; idx++) {
        ui->cbbParam->addItem(val[idx]);
    }
}

void ParamDlg::setToolTip(QString tip) {
    if (cbox) {
        ui->cbbParam->setToolTip(tip);
    } else {
        ui->leParam->setToolTip(tip);
    }

}
