/*FiSysMgr - Utiltiy gui program to help in editing /etc/fstab file.
  Copyright (C) 2015 Barry Strong bstrong@softtechok.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "tablemodel.h"

TableModel::TableModel(QObject *parent, int numcols) : QAbstractTableModel(parent) {
    numRows = 0;
    numcolumns = numcols;
    fstabField = new QStringList [numcols];
    for (int idx = 0; idx < numcolumns; idx++)
        Headers.append("");
}

TableModel::~TableModel() {
    delete [] fstabField;
}

int TableModel::columnCount(const QModelIndex &) const {
    return numcolumns;
}

int TableModel::rowCount(const QModelIndex &) const {
    return numRows;
}

QModelIndex TableModel::index(int row, int column, const QModelIndex &) const {
    return createIndex(row, column);
}

QVariant TableModel::data(const QModelIndex &index, int role) const {
    if (role == Qt::DisplayRole) {
        return fstabField[index.column()][index.row()];
    }
    return QVariant();
}

QVariant TableModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if (role == Qt::DisplayRole) {
        if (orientation == Qt::Horizontal) {
            return Headers.value(section, "");
        }
    }
    return QVariant();
}

bool TableModel::setData(const QModelIndex &index, const QVariant &value, int role) {

    int updCol;
    int updRow;

    if (role == Qt::EditRole) {
        updCol = index.column();
        updRow = index.row();
        fstabField[updCol][updRow] = value.toString();
        emit dataChanged(createIndex(updRow, updCol), createIndex(updRow, updCol));
    }
    return true;
}

bool TableModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value,
                               int role) {
    if (role == Qt::EditRole) {
        if (orientation == Qt::Horizontal) {
            Headers[section] = value.toString();
            emit headerDataChanged(Qt::Horizontal, section, section);
        }
    }
    return true;
}

bool TableModel::insertRows(int row, int count, const QModelIndex &parent) {
    if (count < 1)
        return true;
    beginInsertRows(parent, row, row + count- 1);
    for (int idx = 0; idx < count; idx++) {
        for (int idy = 0; idy < numcolumns; idy++) {
            fstabField[idy].insert(row + idx, "");
        }
    }
    numRows += 1;
    endInsertRows();
    return true;
}

bool TableModel::removeRows(int row, int count, const QModelIndex &parent) {
    if (count < 1)
        return true;
    beginRemoveRows(parent, row, row + count - 1);
    for (int idx = 0; idx < count; idx++) {
        if (numRows > 0) {
            for (int colNum = 0; colNum < numcolumns; colNum++) {
                fstabField[colNum].removeAt(row + idx);
            }
        }
        numRows -= 1;
    }
    endRemoveRows();
    return true;
}
